#include <stdio.h>
#include <string.h>
#include <windows.h>
#include "testrc.h"
#include "blddecli.h"


BOOL CALLBACK _export DlgProc(HWND, UINT, WPARAM, LPARAM);

HINSTANCE   ThisInstance;

#ifdef __BORLANDC__
    #pragma argsused
#endif
int PASCAL WinMain(HINSTANCE Me, HINSTANCE Previous,
		     LPSTR lpszCmdLine, int nCmdShow)
    {
    ThisInstance    = Me;
    HRSRC Dialog = FindResource(ThisInstance, "TEST_DIALOG", RT_DIALOG);
    if(Dialog == NULL)
        {
        MessageBox(NULL, "Could not load main dialog "
            "window!!!", "Wuiman", MB_OK);
        }
    else
        DialogBox(Me, "TEST_DIALOG", NULL, (DLGPROC)DlgProc);
    return 0;
    }

ADdeConversation *Conversation=NULL;


static void EnableButton(HWND Dialog, int Id, int On=TRUE)
    {
    EnableWindow(GetDlgItem(Dialog, Id), On);
    }

static void Error(HWND Dialog, const char *Message)
    {
    MessageBox(Dialog, Message, "DDE Test Error", MB_OK);
    }

static void Connect(HWND Dialog)
    {
    // conversation should not already be in place
    if(Conversation != NULL)
        return;
    char    Service[256];
    char    Topic  [256];
    Service[0]  = Topic[0]  = '\0';
    GetDlgItemText(Dialog, ID_SERVICE_EDIT, Service, sizeof(Service));
    GetDlgItemText(Dialog, ID_TOPIC_EDIT, Topic, sizeof(Topic));
    if(strlen(Service) == 0)
        {
        Error(Dialog, "Enter a service name before connecting.");
        return;
        }
    if(strlen(Topic) == 0)
        {
        Error(Dialog, "Enter a topic name before connecting.");
        return;
        }
    Conversation    = TDdeClient::NewConversation(Service,
                                            Topic, Service);
    if(Conversation == NULL)
        Error(Dialog, "Could not connect!");
    else
        {
        EnableButton(Dialog, ID_CONNECT_BUTTON, FALSE);
        EnableButton(Dialog, ID_DISCONNECT_BUTTON);
        }
    }

static void Disconnect(HWND Dialog)
    {
    if(Conversation != NULL)
        {
        delete Conversation;
        Conversation    = NULL;
        }
    SetDlgItemText(Dialog, ID_RESULT_EDIT, "");
    SetDlgItemText(Dialog, ID_ITEM_EDIT, "");
    SetDlgItemText(Dialog, ID_STATUS_STATIC, "");
    EnableButton(Dialog, ID_DISCONNECT_BUTTON, FALSE);
    EnableButton(Dialog, ID_EXECUTE_BUTTON, FALSE);
    EnableButton(Dialog, ID_REQUEST_BUTTON, FALSE);
    EnableButton(Dialog, ID_POKE_BUTTON, FALSE);
    EnableButton(Dialog, ID_CONNECT_BUTTON);
    }

static void Request(HWND Dialog)
    {
    SetDlgItemText(Dialog, ID_RESULT_EDIT, "");
    if(Conversation == NULL)
        return;
    char Item[256];
    char Result[1024];
    GetDlgItemText(Dialog, ID_ITEM_EDIT, Item, sizeof(Item));
    int Status = Conversation->Request(Item, Result, sizeof(Result));
    SetDlgItemText(Dialog, ID_RESULT_EDIT, Result);
    sprintf(Result, "%d", Status);
    SetDlgItemText(Dialog, ID_STATUS_STATIC, Result);
    }

static void Execute(HWND Dialog)
    {
    SetDlgItemText(Dialog, ID_STATUS_STATIC, "");
    if(Conversation == NULL)
        return;
    char Item[256];
    char Result[1024];
    GetDlgItemText(Dialog, ID_ITEM_EDIT, Item, sizeof(Item));
    int Status = Conversation->Execute(Item);
    sprintf(Result, "%d", Status);
    SetDlgItemText(Dialog, ID_STATUS_STATIC, Result);
    }

static void Poke(HWND Dialog)
    {
    if(Conversation == NULL)
        return;
    char Item[256];
    char Result[1024];
    GetDlgItemText(Dialog, ID_RESULT_EDIT, Result, sizeof(Result));
    GetDlgItemText(Dialog, ID_ITEM_EDIT, Item, sizeof(Item));
    int Status = Conversation->Poke(Item, Result);
    sprintf(Result, "%d", Status);
    SetDlgItemText(Dialog, ID_STATUS_STATIC, Result);
    }

static
BOOL    Command(HWND Dialog, int ControlId, int Command, HWND /*Control*/)
    {
    switch(ControlId)
        {
        case    ID_SERVICE_EDIT :
            if(Command == EN_CHANGE)
                EnableButton(Dialog, ID_CONNECT_BUTTON);
            return FALSE;
        case    ID_ITEM_EDIT :
            if(Command == EN_CHANGE)
                {
                EnableButton(Dialog, ID_REQUEST_BUTTON);
                EnableButton(Dialog, ID_EXECUTE_BUTTON);
                EnableButton(Dialog, ID_POKE_BUTTON);
                }
            return FALSE;
        case    ID_CONNECT_BUTTON :
            if(Command == BN_CLICKED)
                Connect(Dialog);
            return FALSE;
        case    ID_DISCONNECT_BUTTON :
            if(Command == BN_CLICKED)
                Disconnect(Dialog);
            return FALSE;
        case    ID_REQUEST_BUTTON :
            if(Command == BN_CLICKED)
                Request(Dialog);
            return FALSE;
        case    ID_EXECUTE_BUTTON :
            if(Command == BN_CLICKED)
                Execute(Dialog);
            return FALSE;
        case    ID_POKE_BUTTON :
            if(Command == BN_CLICKED)
                Poke(Dialog);
            return FALSE;
        default :
            return FALSE;
        }
    }


#ifdef __BORLANDC__
    #pragma argsused
#endif
BOOL CALLBACK _export DlgProc(HWND Dialog, UINT Message,
                             WPARAM Param1, LPARAM Param2)
    {
    if(Message == WM_INITDIALOG)
        {
        return TRUE;
        }
    else if(Message == WM_COMMAND)
        if(Param1 == IDOK || Param1 == IDCANCEL) {
            EndDialog(Dialog, 0);
            return TRUE;
            }
        else
            return Command(Dialog, (int)Param1,
                      HIWORD(Param2), (HWND)LOWORD(Param2));
    else if(Message == WM_DESTROY)
        Disconnect(Dialog);
    return FALSE;
    }


